"""
conftest.py

Contains PyTest fixtures used throughout tests.

Created by Stephen Andrews, September 18th, 2018.
"""

import psycopg2
import pytest

from app import create_app
from app.extensions import db


@pytest.fixture(scope='module')
def test_client():
    flask_app = create_app()

    # Flask provides a way to test your application by exposing the Werkzeug test Client
    # and handling the context locals for you.
    testing_client = flask_app.test_client()

    # Establish an application context before running the tests.
    ctx = flask_app.app_context()
    ctx.push()

    # Execute tests
    yield testing_client

    ctx.pop()

@pytest.fixture(scope='module')
def init_database(request):
    """
    Initializes a database connection for testing. All tables are created
    and dropped when using this fixture.

    :param request: An optional list of models to populate the database with. Use
                    indirect parametrization to pass data.
    """
    # Create the database and the database table
    db.create_all()

    # Insert data
    try:
        for model in request.param:
            db.session.add(model)
            db.session.commit()
    except AttributeError:
        pass

    # Execute tests
    yield db

    # Explicitly close DB connection (https://stackoverflow.com/a/31867087)
    db.session.close()

    db.drop_all()
