"""
google_books_api.py

Created by Stephen Andrews, October 25th, 2018.
"""

import json
import re
from collections import namedtuple
from typing import List

import requests

class GoogleBook:
    """Represents a book from the Google Books API."""


class GoogleBooks:
    """Python wrapper for the Google books API."""
    
    # Base URL of Google books API
    API_BASE = 'https://www.googleapis.com/books/v1/volumes?q={type}:{search_term}'

    ISBN_10_LENGTH = 10
    ISBN_13_LENGTH = 13

    def __init__(self):
        pass

    def is_isbn(self, text: str) -> bool:
        """
        Determines whether the specified text is an ISBN.

        :param text: The text to test.
        :return: A boolean indicating whether or not the text is an ISBN.
        """
        if (len(text) != self.ISBN_10_LENGTH and len(text) != self.ISBN_13_LENGTH):
            return False

        r_isbn = r'\d{10}|\d{13}'

        return re.match(r_isbn, text)

    def search_isbn(self, isbn: str) -> dict:
        """Get a book by its ISBN."""
        endpoint = self.API_BASE.format(type='isbn', search_term=isbn)
        response = requests.get(endpoint)
        data = response.json()['items']

        if len(data) <= 0:
            return {}

        return self.parse_response(data[0])

    def suggest(self, search_term: str) -> List[dict]:
        endpoint = self.API_BASE.format(type='intitle', search_term=search_term)
        
        # Check if user is searching by ISBN
        if self.is_isbn(search_term):
            endpoint = self.API_BASE.format(type='isbn', search_term=search_term)

        response = requests.get(endpoint)
        suggestions = []
        if response.json()['totalItems'] > 0:
            for item in response.json()['items']:
                book = self.parse_response(item)

                if book:
                    suggestions.append(book)

        return suggestions

    def parse_response(self, data: dict) -> dict:
        """
        Parses a json response of a book into the Book named tupled.

        :param data: A json representation of a single book returned from the API.
        :return: An optional containing a named tuple representing the book.
        """
        try:
            # ISBN ordering is random
            isbn_10, isbn_13 = None, None
            first_isbn_type = data['volumeInfo']['industryIdentifiers'][0]['type']
            
            if first_isbn_type == 'ISBN_10':
                isbn_10 = data['volumeInfo']['industryIdentifiers'][0]['identifier']
                isbn_13 = data['volumeInfo']['industryIdentifiers'][1]['identifier']
            else:
                isbn_13 = data['volumeInfo']['industryIdentifiers'][0]['identifier']
                isbn_10 = data['volumeInfo']['industryIdentifiers'][1]['identifier']

            book = {
                'title': data['volumeInfo']['title'],
                'subtitle': data['volumeInfo'].get('subtitle', None),
                'authors': data['volumeInfo']['authors'],
                'isbn_10': isbn_10,
                'isbn_13': isbn_13,
                'publisher': data['volumeInfo']['publisher'],
                'published_date': data['volumeInfo']['publishedDate'],
                'categories': data['volumeInfo'].get('categories', None),
                'description': data['volumeInfo'].get('description', None),
                'small_thumbnail': data['volumeInfo']['imageLinks']['smallThumbnail'],
                'thumbnail': data['volumeInfo']['imageLinks']['thumbnail']
            }

            # Inconsistent published date we only want the year
            published_date = re.match(r'\d{4}', book['published_date']).group(0)
            book['published_date'] = published_date

            # Get elements of categories array if present
            categories = book['categories']
            if categories:
                book['categories'] = categories[0]
            
            return book
        except Exception as e:
            print('[google_books_api] parse_response error')
            print(e)
            return None
