import {Permissions, Notifications} from 'expo'; // Need for push notifications
import * as Constants from './Constants';
import StorageManager from './StorageManager';

/**
 * Base URL of API.
 */
const BASE_URL = Constants.DEV_MODE ? Constants.DEV_API : Constants.API;

let createListing = async (seller_id, book_id, condition, description, callback) => {
  let url = BASE_URL + "/listing/post";
  let method = "POST";
  let payload = {
    'seller_id': seller_id,
    'book_id': book_id,
    'condition': condition,
    'description': description
  };
  try {
    let result = await fetch(url, {
      'method': method,
      'body': JSON.stringify(payload),
      'headers': {
        'Content-Type': 'application/json'
      }
    });
    callback(result);
  } catch (error) {
    console.error("api.js error in createListing: " + error);
  }
}

getSessionID = async (callback) => {
  let session_id = null;
  let cb = async(result) => {
    session_id = result;
    await callback(session_id);
  }
  await StorageManager.getItem('session_id', cb);
}

const API = {
  loginUser: async (username, password, callback) => {
    let url = BASE_URL + "/user/login";
    let payload = {
      'username': username,
      'password': password
    };
    let method = "POST";
    
    try {
      let result = await fetch(url, {
        'method': method,
        'body': JSON.stringify(payload),
        headers: {
          'Content-Type': 'application/json'
        }
      });
      callback(result);
    } catch (error) {
      console.error("api.js error in loginUser: " + error);
    }
  },
  registerUser: async (username, password, universityId, email, callback) => {
    // TODO: should probably put this in a different function or class
    let url = BASE_URL + "/user/register";
    let payload = {
      'username': username,
      'password': password,
      'university_id': universityId,
      'email': email
    };
    let method = "POST";

    try {
      let result = await fetch(url, {
        'method': method,
        'body': JSON.stringify(payload),
        headers: {
          'Content-Type': 'application/json'
        }
      });
      callback(result);
    } catch (error) {
      console.error("api.js error in registerUser: " + error);
    }
  },
  getListedBooks: async (callback, params=null, page, count) => {
    // let endpoint = params != null ? BASE_URL + '/listing/?' + params :  BASE_URL + '/listing/';
    let endpoint = BASE_URL + '/listing/';

    if(typeof page !== "undefined") {
      endpoint = endpoint + "&p="+page;
    }
    if(typeof count !== "undefined") {
      endpoint = endpoint + "&c="+count;
    }

    if(params != null) {
      endpoint += '?'+params;
    }
    
    try {
      this.getSessionID(async(sessionID) =>  {
        
        let result = await fetch(endpoint, {
          'headers': {
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in getListedBooks: " + error);
    }
  },

  /**
   * Get all the listings for a specified book ID.
   * @param bookId The id of the book to get the listings for.
   * @param callback The callback function containing the API response data.
   * @param params Optional parameteres used for filtering.
   */
  getListings: async (bookId, callback, params=null, page, count) => {
    let endpoint = BASE_URL + '/listing/' + bookId;

    if (typeof page !== "undefined") {
      endpoint = endpoint + "&p="+page;
    }
    if (typeof count !== "undefined") {
      endpoint = endpoint + "&c="+count;
    }

    if(params != null) {
      endpoint += '?' + params;
    }

    try {
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(endpoint, {
          headers: {
            'Authorization': 'Bearer ' + sessionID
          }
        });

        callback(result);
      });
    } catch (error) {
      console.log("[API] Error getting listings for book ID: " + bookId);
      console.error(error);
    }
  },

  /**
   * Create a new sale type listings.
   * @param seller_id The user ID of the seller.
   * @param book_id The book's ID.
   * @param condition A string representing the quality of the book.
   * @param description Extra information provided by user.
   * @param price The cost of the book.
   */
  createSale: async (seller_id, book_id, condition, description, price, photoURLs, callback) => {
    let endpoint = BASE_URL + '/listing/post';
    let method = "POST";
    let payload = {
      'seller_id': seller_id,
      'book_id': book_id,
      'condition': condition,
      'description': description,
      'price': price,
      'photos': photoURLs
    };

    try {
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(endpoint, {
          'method': method,
          'body': JSON.stringify(payload),
          'headers': {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in createSale: " + error);
    }
  },
  
  /**
   * Puts a listing in an inactive state and removes it from all visible listings.
   */
  removeListing: async(listing_id, callback) => {
    let endpoint = BASE_URL + '/listing/remove/' + listing_id;

    try {
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(endpoint, {
          'method': "DELETE",
          'headers': {
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in removeListing: " + error);
    }
  },
  getBookIDFromISBN: async (isbn, callback) => {
    let endpoint = BASE_URL + '/book/' + isbn;
    try {
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(endpoint, {
          headers: {
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in getBookIDFromISBN: " + error);
    }
  },
  resetUser: async (email, callback) => {
    let url = BASE_URL + "/user/forgot/" + email;
    try{
      let result = await fetch(url);
      callback(result);
    } catch (error) {
      console.error("api.js error in resetUser: " + error);
    }
  },
  getOwned: async(user_id, callback, page, count) => {
    let url = BASE_URL + "/library/" + user_id;
    if(typeof page !== "undefined") {
      url = url + "&p="+page;
    }
    if(typeof count !== "undefined") {
      url = url + "&c="+count;
    }
    try{
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(url, {
          headers: {
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in getOwned: " + error);
    }
  },
  addOwned: async(user_id, book_id, callback) => {
    let url = BASE_URL + "/library/add";
    let method = "POST";
    let payload = {
      "user_id":user_id,
      "book_id":book_id
    };
    try{
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(url, {
          'method': method,
          'body': JSON.stringify(payload),
          'headers': {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in addOwned: " + error);
    }
  },
  getMessageThreads: async(user_id, callback) => {
    let url = BASE_URL + "/messages/" + user_id;
    try{
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(url, {
          method: 'GET',
          headers: {
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in getMessageThreads: " + error);
    }
  },
  getConversation: async(thread_id, callback, page, count) => {
    let url = BASE_URL + "/messages/thread/" + thread_id;
    
    if(typeof page !== "undefined") {
      url = url + "&p="+page;
    }
    if(typeof count !== "undefined") {
      url = url + "&c="+count;
    }

    try{
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(url, {
          headers: {
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in getConversation: " + error);
    }
  },
  globalSearchTitle: async(title, callback) => {
    let url = BASE_URL + "/book/search/" + title;
    let method = "GET";
  
    try{
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(url, {
          'method': method,
          'headers': {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in addOwned: " + error);
    }
  },
  sendMessage: async(from_user_id, to_user_id, message, callback) => {
    let url = BASE_URL + "/messages/send";
    let method = "POST";
    let payload = {
      "from_user": from_user_id,
      "to_user": to_user_id,
      "message": message,
      "is_request": false
    };
    try{
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(url, {
          method: method,
          body: JSON.stringify(payload),
          headers: {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in sendMessage: " + error);
    }
  },
  // Push notifications
  registerForPushNotifications: async(user_id) => {
    const {status: existingStatus} = await Permissions.getAsync(
      Permissions.NOTIFICATIONS
    );
    let finalStatus = existingStatus;

    // only ask if permissions have not already been determined
    // because ios might not prompt a second time
    if(existingStatus !== 'granted') {
      // android grants permissions on install
      // this only asks ios
      const {status} = await Permissions.askAsync(Permissions.NOTIFICATIONS);
      finalStatus = status;
    }

    
    // Stop if user did not grant permissions
    if(finalStatus !== 'granted') {
      return;
    }
    // get the expo token
    let token = await Notifications.getExpoPushTokenAsync();
    console.log(token);
    // POST the token to the backend
    // requires user id 'user_id' and expo token 'token'
    let url = 'https://bookstand.io/notifications/add';
    let payload = {
      expo_token: token,
      user_id: user_id
    };
    let method = 'POST';
    return fetch(url, {
      method: method,
      headers: {
        Accept: 'application/json',
        'Content-Type':'application/json',
      },
      body: JSON.stringify(payload)
    });
  },
  getThreadByUserIDs: async(user_id1, user_id2, callback) => {
    let url = BASE_URL + "/messages/"+user_id1+"&"+user_id2;

    try{
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(url, {
          headers: {
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in getThreadByUserIDs: " + error);
    }
  },
  uploadPhotos: async (photos, username, ifListing, callback, urlCallback) => {
    // create a listing then use the listing id in the return to create a sale
    let url = BASE_URL + "/image/upload";
    let method = "POST";
    const photoData = new FormData();
    // for identifying images
    let date = new Date();
    var day = date.getDate();
    var month = date.getMonth() + 1;
    var year = date.getFullYear();
    var hours = date.getHours();
    var minutes = date.getMinutes();
    var sec = date.getSeconds();
    let i = 0;
    let keyfrag = username + '/d' + day + "/m" + month + "/y" + year + "/h" + hours + "/m" + minutes + "/s" + sec + "/";
    let photoURLs = [];
    photos.forEach((photo) => {
      let key = 'photo/' + (ifListing ? "listing/" : "profile/") + keyfrag + (i++) + '.jpeg';
      photoData.append(key, {
        uri: photo.uri,
        name: 'photo',
        type: 'image/jpeg'
      });
      photoURLs.push("https://s3.amazonaws.com/bookstand/" + key);
    });
    urlCallback(photoURLs);
    try{
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(url, {
          'method': method,
          'body': photoData,
          'headers': {
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in uploadPhotos: " + error.message);
    }
  },
  removeOwned: async(id, callback) => {
    let url = "https://bookstand.io/library/remove/" + id;
    let method = 'DELETE';

    try{
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(url, {
          method: method,
          headers: {
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in removeOwned: " + error);
    }
  },
  updateProfilePicture: async (username, pictureURL, callback) => {
    let endpoint = BASE_URL + '/user/picture';
    let method = "POST";
    let payload = {
      'username': username,
      'picture': pictureURL
    };

    try { 
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(endpoint, {
          'method': method,
          'body': JSON.stringify(payload),
          'headers': {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in updateProfilePicture: " + error);
    }
  },
  submitBugReport: async (username, report, callback) => {
    let url = BASE_URL + "/user/bugreport/";
    let payload = {
      'username': username,
      'report': report
    }

    try {
      this.getSessionID(async(sessionID) => {
        let result = await fetch(url, {
          'method': "POST",
          'body': JSON.stringify(payload),
          headers: {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + sessionID
          }
        });

        callback(result);
      });
    } catch (error) {
      console.error("api.js error in submitBugReport: " + error);
    }
  },
  changePassword: async (username, old_password, new_password, callback) => {
    let url = BASE_URL + "/user/changepw/";
    let payload = {
      'username': username,
      'oldpw': old_password,
      'newpw': new_password
    };
    let method = "POST";
    try {
      this.getSessionID(async(sessionID) => {
        let result = await fetch(url, {
          'method': method,
          'body': JSON.stringify(payload),
          headers: {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in changePassword: " + error);
    }
  },
  // Get a user's active listings
  getActiveListings: async(username, callback) => {
    let endpoint = BASE_URL + '/user/listings/' + username;
    
    try { 
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(endpoint, {
          'headers': {
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in getActiveListings: " + error);
    }
  },
  getTransactionHistory:  async(user_id, callback) => {
    let endpoint = BASE_URL + '/transactions/user/' + user_id;
    try { 
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(endpoint, {
          'headers': {
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in updateProfilePicture: " + error);
    }
  },
  addTransaction: async(listing_id, buyer_id, callback) => {
    let endpoint = BASE_URL + '/transactions/';
    let payload = {
      "listing_id": listing_id,
      "buyer_id": buyer_id
    };
    let method = 'POST';

    try { 
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(endpoint, {
          'method': method,
          'body': JSON.stringify(payload),
          'headers': {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in updateProfilePicture: " + error);
    }
  },
  // For a trade request, we use the messaging endpoint
  // The listing_id is used as the message to indicate which listing the person wants to buy a book from
  sendTradeRequest: async(from_user_id, to_user_id, listing_id, callback) => {
    let url = BASE_URL + "/messages/send";
    let method = "POST";
    let payload = {
      "from_user": from_user_id,
      "to_user": to_user_id,
      "message": listing_id,
      "is_request": true
    };
    try{
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(url, {
          method: method,
          body: JSON.stringify(payload),
          headers: {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in sendTradeRequest: " + error);
    }
  },
  // transaction_id is optional. if it is null, the payload won't include it
  // must include a callback
  // status is a string
  sendTransaction: async(buyer_user_id, listing_id, status, transaction_id, callback) => {
    let url = BASE_URL + "/transactions/";
    let method = "POST";

    if(callback == null) {
      callback = transaction_id;
      transaction_id = null;
    }
    let payload = {};
    if(transaction_id == null) {
      payload = {
        "buyer_id": buyer_user_id,
        "listing_id": listing_id,
        "status": status
      };
    } else {
      payload = {
        "buyer_id": buyer_user_id,
        "listing_id": listing_id,
        "status": status,
        "transaction_id": transaction_id
      };
    }
    
    try{
      this.getSessionID(async(sessionID) =>  {
        let result = await fetch(url, {
          method: method,
          body: JSON.stringify(payload),
          headers: {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + sessionID
          }
        });
        callback(result);
      });
    } catch (error) {
      console.error("api.js error in sendTradeRequest: " + error);
    }
  },

  /**
   * Add a user to the interested list for non-WPI emails.
   * @param email The email they attempted to register with.
   */
  addInterested: async(email, callback) => {
    let url = BASE_URL + '/interested/';
    
    try {
      this.getSessionID(async(sessionID) => {
        let result = await fetch(url, {
          method: "POST",
          body: JSON.stringify({email: email}),
          headers: {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + sessionID
          }
        });

        callback(result);
      })
    } catch (error) {
      console.log("api.js error in addInterested: " + error);
    }
  }

}

export default API;
